@@ -36,6 +36,8 @@ pp_macro_defined_unterminated
 pp_macro_expansion
 pp_macro_expansion_multiline
 pp_macro_inline_unterminated
+pp_macro_missing_expression
+pp_macro_unexpected_tokens
 pp_macro_nargs
 pp_macro_redef
 pp_macro_rparen
@@ -56,26 +58,33 @@ swig_identifier
 swig_insert_bad
 swig_typemap_copy
 swig_typemap_old
+swig_typemap_warn
 '
 
 # Files run in C++ mode
 CPPFILES='
 cpp_bad_extern
+cpp_extend_destructors
 cpp_extend_redefine
 cpp_extend_undefined
 cpp_inline_namespace
 cpp_inherit
 cpp_macro_locator
+cpp_missing_rparenthesis
 cpp_missing_rtemplate
 cpp_namespace_alias
 cpp_namespace_aliasnot
 cpp_namespace_aliasundef
 cpp_nested
 cpp_no_access
+cpp_no_return_type
 cpp_nobase
 cpp_overload
+cpp_overload_const
 cpp_private_defvalue
 cpp_private_inherit
+cpp_recursive_typedef
+cpp_shared_ptr
 cpp_template_argname
 cpp_template_nargs
 cpp_template_not
@@ -98,14 +107,14 @@ for i in ${CFILES}; do
    echo "    Testing : ${i}.i"; 
    echo "" >> ${LOGFILE};
    echo ":::::::::::::::::::::::::::::::: ${i}.i :::::::::::::::::::::::::::::::::::" >> ${LOGFILE};
-   ${SWIG} -Wall ${SWIGOPT} ${i}.i >>${LOGFILE} 2>&1
+   ${SWIG} -python -Wall ${SWIGOPT} ${i}.i >>${LOGFILE} 2>&1
 done
 
 for i in ${CPPFILES}; do 
    echo "    Testing : ${i}.i"; 
    echo "" >> ${LOGFILE}
    echo ":::::::::::::::::::::::::::::::: ${i}.i :::::::::::::::::::::::::::::::::::" >> ${LOGFILE};
-   ${SWIG} -Wall -c++ ${SWIGOPT} ${i}.i >>${LOGFILE} 2>&1
+   ${SWIG} -python -Wall -c++ ${SWIGOPT} ${i}.i >>${LOGFILE} 2>&1
 done
 
 echo ""
