@@ -9,12 +9,14 @@
  * scanner.c
  *
  * SWIG tokenizer.  This file is a wrapper around the generic C scanner
- * found in Swig/scanner.c.   Extra logic is added both to accommodate the
+ * found in Swig/scanner.c.   Extra logic is added both to accomodate the
  * bison-based grammar and certain peculiarities of C++ parsing (e.g.,
  * operator overloading, typedef resolution, etc.).  This code also splits
  * C identifiers up into keywords and SWIG directives.
  * ----------------------------------------------------------------------------- */
 
+char cvsroot_cscanner_c[] = "$Id: cscanner.c 13976 2012-12-16 13:56:50Z wsfulton $";
+
 #include "cparse.h"
 #include "parser.h"
 #include <string.h>
@@ -371,7 +373,7 @@ void scanner_clear_rename() {
   rename_active = 0;
 }
 
-/* Used to push a fictitious token into the scanner */
+/* Used to push a ficticious token into the scanner */
 static int next_token = 0;
 void scanner_next_token(int tok) {
   next_token = tok;
