@@ -12,6 +12,8 @@
  *     formatted output, readline, and splitting.
  * ----------------------------------------------------------------------------- */
 
+char cvsroot_fio_c[] = "$Id: fio.c 13969 2012-12-15 14:55:54Z wsfulton $";
+
 #include "dohint.h"
 
 #define OBUFLEN  512
@@ -21,7 +23,7 @@ static DOH *encodings = 0;	/* Encoding hash */
 /* -----------------------------------------------------------------------------
  * Writen()
  *
- * Writes N characters of output and retries until all characters are
+ * Write's N characters of output and retries until all characters are
  * written.  This is useful should a write operation encounter a spurious signal.
  * ----------------------------------------------------------------------------- */
 
