@@ -11,7 +11,7 @@
  * CHICKEN language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_chicken_cxx[] = "$Id: chicken.cxx 12830 2011-10-30 21:51:50Z wsfulton $";
+char cvsroot_chicken_cxx[] = "$Id: chicken.cxx 13885 2012-11-14 22:16:07Z wsfulton $";
 
 #include "swigmod.h"
 
@@ -290,8 +290,6 @@ int CHICKEN::top(Node *n) {
 
   Printf(f_scm, "%s\n", chickentext);
 
-
-  Close(f_scm);
   Delete(f_scm);
 
   char buftmp[20];
@@ -324,7 +322,6 @@ int CHICKEN::top(Node *n) {
   Delete(f_wrappers);
   Delete(f_sym_size);
   Delete(f_init);
-  Close(f_begin);
   Delete(f_runtime);
   Delete(f_begin);
   return SWIG_OK;
