@@ -13,6 +13,8 @@
  * in SWIG.  --MR
  * ----------------------------------------------------------------------------- */
 
+char cvsroot_directors_cxx[] = "$Id";
+
 #include "swigmod.h"
 
 /* Swig_csuperclass_call()
@@ -76,10 +78,9 @@ String *Swig_class_name(Node *n) {
 
 String *Swig_director_declaration(Node *n) {
   String *classname = Swig_class_name(n);
-  String *directorname = Language::instance()->directorClassName(n);
+  String *directorname = NewStringf("SwigDirector_%s", classname);
   String *base = Getattr(n, "classtype");
   String *declaration = Swig_class_declaration(n, directorname);
-
   Printf(declaration, " : public %s, public Swig::Director {\n", base);
   Delete(classname);
   Delete(directorname);
@@ -279,10 +280,9 @@ void Swig_director_emit_dynamic_cast(Node *n, Wrapper *f) {
                                                checkAttribute(n, "storage", "static"))
                                           && !Equal(nodeType(n), "constructor"))) {
     Node *parent = Getattr(n, "parentNode");
-    String *dirname;
-    String *dirdecl;
-    dirname = Language::instance()->directorClassName(parent);
-    dirdecl = NewStringf("%s *darg = 0", dirname);
+    String *symname = Getattr(parent, "sym:name");
+    String *dirname = NewStringf("SwigDirector_%s", symname);
+    String *dirdecl = NewStringf("%s *darg = 0", dirname);
     Wrapper_add_local(f, "darg", dirdecl);
     Printf(f->code, "darg = dynamic_cast<%s *>(arg1);\n", dirname);
     Delete(dirname);
