@@ -11,7 +11,7 @@
  * Useful functions for emitting various pieces of code.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_emit_cxx[] = "$Id: emit.cxx 13511 2012-08-05 08:37:59Z oliv3rb $";
+char cvsroot_emit_cxx[] = "$Id: emit.cxx 13871 2012-10-11 19:28:02Z wsfulton $";
 
 #include "swigmod.h"
 
@@ -363,24 +363,9 @@ int emit_action_code(Node *n, String *wrappercode, String *eaction) {
     tm = Copy(tm);
   if ((tm) && Len(tm) && (Strcmp(tm, "1") != 0)) {
     if (Strstr(tm, "$")) {
-      Replaceall(tm, "$name", Getattr(n, "name"));
-      Replaceall(tm, "$symname", Getattr(n, "sym:name"));
+      Swig_replace_special_variables(n, parentNode(n), tm);
       Replaceall(tm, "$function", eaction); // deprecated
       Replaceall(tm, "$action", eaction);
-      Replaceall(tm, "$wrapname", Getattr(n, "wrap:name"));
-      String *overloaded = Getattr(n, "sym:overloaded");
-      Replaceall(tm, "$overname", overloaded ? Char(Getattr(n, "sym:overname")) : "");
-
-      if (Strstr(tm, "$decl")) {
-        String *decl = Swig_name_decl(n);
-        Replaceall(tm, "$decl", decl);
-        Delete(decl);
-      }
-      if (Strstr(tm, "$fulldecl")) {
-        String *fulldecl = Swig_name_fulldecl(n);
-        Replaceall(tm, "$fulldecl", fulldecl);
-        Delete(fulldecl);
-      }
     }
     Printv(wrappercode, tm, "\n", NIL);
     Delete(tm);
