@@ -37,7 +37,7 @@
  * (may need to add more WARN_PHP_xxx codes...)
  */
 
-char cvsroot_php_cxx[] = "$Id: php.cxx 13105 2012-05-23 02:58:24Z olly $";
+char cvsroot_php_cxx[] = "$Id: php.cxx 13972 2012-12-15 18:33:02Z wsfulton $";
 
 #include "swigmod.h"
 
@@ -354,7 +354,9 @@ public:
       Printf(f_directors_h, "#ifndef SWIG_%s_WRAP_H_\n", cap_module);
       Printf(f_directors_h, "#define SWIG_%s_WRAP_H_\n\n", cap_module);
 
-      Printf(f_directors, "\n#include \"%s\"\n\n", Swig_file_filename(outfile_h));
+      String *filename = Swig_file_filename(outfile_h);
+      Printf(f_directors, "\n#include \"%s\"\n\n", filename);
+      Delete(filename);
     }
 
     /* PHP module file */
@@ -615,7 +617,7 @@ public:
 
     Printf(f_h, "#endif /* PHP_%s_H */\n", cap_module);
 
-    Close(f_h);
+    Delete(f_h);
 
     String *type_table = NewStringEmpty();
     SwigType_emit_type_table(f_runtime, type_table);
@@ -630,7 +632,7 @@ public:
       Dump(f_directors_h, f_runtime_h);
       Printf(f_runtime_h, "\n");
       Printf(f_runtime_h, "#endif\n");
-      Close(f_runtime_h);
+      Delete(f_runtime_h);
     }
 
     Printf(s_header, "/* end header section */\n");
@@ -654,7 +656,6 @@ public:
     Delete(s_vdecl);
     Delete(all_cs_entry);
     Delete(s_entry);
-    Close(f_begin);
     Delete(f_runtime);
     Delete(f_begin);
 
@@ -667,7 +668,7 @@ public:
       s_fakeoowrappers = NULL;
     }
     Printf(f_phpcode, "%s\n?>\n", s_phpclasses);
-    Close(f_phpcode);
+    Delete(f_phpcode);
 
     return SWIG_OK;
   }
@@ -1090,9 +1091,6 @@ public:
       int min_num_of_arguments = emit_num_required(l);
       int max_num_of_arguments = emit_num_arguments(l);
 
-      // For a function with default arguments, we end up with the fullest
-      // parmlist in full_parmlist.
-      ParmList *full_parmlist = l;
       Hash *ret_types = NewHash();
       Setattr(ret_types, d, d);
 
@@ -1125,7 +1123,6 @@ public:
 
 	  if (num_arguments > max_num_of_arguments) {
 	    max_num_of_arguments = num_arguments;
-	    full_parmlist = l2;
 	  }
 	  o = Getattr(o, "sym:nextSibling");
 	}
@@ -1176,11 +1173,6 @@ public:
 	if (!o) {
 	  // This "overloaded method" is really just one with default args.
 	  really_overloaded = false;
-	  if (l != full_parmlist) {
-	    l = full_parmlist;
-	    if (wrapperType == memberfn)
-	      l = nextSibling(l);
-	  }
 	}
       }
 
@@ -1581,7 +1573,8 @@ public:
 	  while (i.item) {
 	    Node *j = firstChild(i.item);
 	    while (j) {
-	      if (Strcmp(Getattr(j, "name"), Getattr(n, "name")) != 0) {
+	      String *jname = Getattr(j, "name");
+	      if (!jname || Strcmp(jname, Getattr(n, "name")) != 0) {
 		j = nextSibling(j);
 		continue;
 	      }
@@ -1702,9 +1695,9 @@ public:
 	  Printf(output, "\t\t$r=%s;\n", invoke);
 	if (Len(ret_types) == 1) {
 	  /* If d is abstract we can't create a new wrapper type d. */
-	  Node * d_class = classLookup(d);
+	  Node *d_class = classLookup(d);
 	  int is_abstract = 0;
-	  if (Getattr(d_class, "abstract")) {
+	  if (Getattr(d_class, "abstracts")) {
 	    is_abstract = 1;
 	  }
 	  if (newobject || !is_abstract) {
@@ -2024,7 +2017,7 @@ done:
 	base.item = NULL;
       }
 
-      if (Getattr(n, "abstract") && !GetFlag(n, "feature:notabstract")) {
+      if (Getattr(n, "abstracts") && !GetFlag(n, "feature:notabstract")) {
 	Printf(s_phpclasses, "abstract ");
       }
 
@@ -2331,6 +2324,7 @@ done:
     Printf(f->code, "}\n");
 
     Wrapper_print(f, s_wrappers);
+    DelWrapper(f);
 
     return SWIG_OK;
   }
@@ -2415,18 +2409,17 @@ done:
   int classDirectorMethod(Node *n, Node *parent, String *super) {
     int is_void = 0;
     int is_pointer = 0;
-    String *decl;
-    String *type;
-    String *name;
-    String *classname;
+    String *decl = Getattr(n, "decl");
+    String *returntype = Getattr(n, "type");
+    String *name = Getattr(n, "name");
+    String *classname = Getattr(parent, "sym:name");
     String *c_classname = Getattr(parent, "name");
     String *symname = Getattr(n, "sym:name");
-    String *declaration;
-    ParmList *l;
-    Wrapper *w;
+    String *declaration = NewStringEmpty();
+    ParmList *l = Getattr(n, "parms");
+    Wrapper *w = NewWrapper();
     String *tm;
     String *wrap_args = NewStringEmpty();
-    String *return_type;
     String *value = Getattr(n, "value");
     String *storage = Getattr(n, "storage");
     bool pure_virtual = false;
@@ -2440,34 +2433,15 @@ done:
       }
     }
 
-    classname = Getattr(parent, "sym:name");
-    type = Getattr(n, "type");
-    name = Getattr(n, "name");
-
-    w = NewWrapper();
-    declaration = NewStringEmpty();
-
     /* determine if the method returns a pointer */
-    decl = Getattr(n, "decl");
     is_pointer = SwigType_ispointer_return(decl);
-    is_void = (Cmp(type, "void") == 0 && !is_pointer);
-
-    /* form complete return type */
-    return_type = Copy(type);
-    {
-      SwigType *t = Copy(decl);
-      SwigType *f = SwigType_pop_function(t);
-      SwigType_push(return_type, t);
-      Delete(f);
-      Delete(t);
-    }
+    is_void = (Cmp(returntype, "void") == 0 && !is_pointer);
 
     /* virtual method definition */
-    l = Getattr(n, "parms");
     String *target;
     String *pclassname = NewStringf("SwigDirector_%s", classname);
     String *qualified_name = NewStringf("%s::%s", pclassname, name);
-    SwigType *rtype = Getattr(n, "conversion_operator") ? 0 : type;
+    SwigType *rtype = Getattr(n, "conversion_operator") ? 0 : Getattr(n, "classDirectorMethods:type");
     target = Swig_method_decl(rtype, decl, qualified_name, l, 0, 0);
     Printf(w->def, "%s", target);
     Delete(qualified_name);
@@ -2490,7 +2464,7 @@ done:
       if (throw_parm_list)
 	Swig_typemap_attach_parms("throws", throw_parm_list, 0);
       for (p = throw_parm_list; p; p = nextSibling(p)) {
-	if ((tm = Getattr(p, "tmap:throws"))) {
+	if (Getattr(p, "tmap:throws")) {
 	  if (gencomma++) {
 	    Append(w->def, ", ");
 	    Append(declaration, ", ");
@@ -2517,7 +2491,7 @@ done:
      */
     if (!is_void) {
       if (!(ignored_method && !pure_virtual)) {
-	String *cres = SwigType_lstr(return_type, "c_result");
+	String *cres = SwigType_lstr(returntype, "c_result");
 	Printf(w->code, "%s;\n", cres);
 	Delete(cres);
       }
@@ -2651,16 +2625,7 @@ done:
 
       /* marshal return value */
       if (!is_void) {
-	/* this seems really silly.  the node's type excludes
-	 * qualifier/pointer/reference markers, which have to be retrieved
-	 * from the decl field to construct return_type.  but the typemap
-	 * lookup routine uses the node's type, so we have to swap in and
-	 * out the correct type.  it's not just me, similar silliness also
-	 * occurs in Language::cDeclaration().
-	 */
-	Setattr(n, "type", return_type);
 	tm = Swig_typemap_lookup("directorout", n, Swig_cresult_name(), w);
-	Setattr(n, "type", type);
 	if (tm != 0) {
 	  static const String *amp_result = NewStringf("&%s", Swig_cresult_name());
 	  Replaceall(tm, "$input", amp_result);
@@ -2679,7 +2644,7 @@ done:
 	  Delete(tm);
 	} else {
 	  Swig_warning(WARN_TYPEMAP_DIRECTOROUT_UNDEF, input_file, line_number,
-	      "Unable to use return type %s in director method %s::%s (skipping method).\n", SwigType_str(return_type, 0), SwigType_namestr(c_classname),
+	      "Unable to use return type %s in director method %s::%s (skipping method).\n", SwigType_str(returntype, 0), SwigType_namestr(c_classname),
 	      SwigType_namestr(name));
 	  status = SWIG_ERROR;
 	}
@@ -2706,8 +2671,8 @@ done:
 
     if (!is_void) {
       if (!(ignored_method && !pure_virtual)) {
-	String *rettype = SwigType_str(return_type, 0);
-	if (!SwigType_isreference(return_type)) {
+	String *rettype = SwigType_str(returntype, 0);
+	if (!SwigType_isreference(returntype)) {
 	  Printf(w->code, "return (%s) c_result;\n", rettype);
 	} else {
 	  Printf(w->code, "return (%s) *c_result;\n", rettype);
@@ -2749,7 +2714,6 @@ done:
 
     /* clean up */
     Delete(wrap_args);
-    Delete(return_type);
     Delete(pclassname);
     DelWrapper(w);
     return status;
