@@ -11,7 +11,7 @@
  * A parse tree represented as Lisp s-expressions.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_s_exp_cxx[] = "$Id: s-exp.cxx 13340 2012-07-21 22:52:30Z wsfulton $";
+char cvsroot_s_exp_cxx[] = "$Id: s-exp.cxx 13940 2012-11-26 20:31:54Z wsfulton $";
 
 #include "swigmod.h"
 #include "dohint.h"
@@ -24,9 +24,21 @@ S-Exp Options (available with -sexp)\n\
 static File *out = 0;
 
 class Sexp:public Language {
-public:
   int indent_level;
-   Sexp():indent_level(0) {
+  DOHHash *print_circle_hash;
+  int print_circle_count;
+  int hanging_parens;
+  bool need_whitespace;
+  bool need_newline;
+
+public:
+  Sexp():
+    indent_level(0),
+    print_circle_hash(0),
+    print_circle_count(0),
+    hanging_parens(0),
+    need_whitespace(0),
+    need_newline(0) {
   }
   
   virtual ~ Sexp() {
@@ -51,12 +63,6 @@ public:
     }
   }
 
-  DOHHash *print_circle_hash;
-  int print_circle_count;
-  int hanging_parens;
-  bool need_whitespace;
-  bool need_newline;
-
   /* Top of the parse tree */
   virtual int top(Node *n) {
     if (out == 0) {
@@ -81,7 +87,7 @@ public:
     Language::top(n);
     Printf(out, "\n");
     Printf(out, ";;; Lisp parse tree produced by SWIG\n");
-    print_circle_hash = DohNewHash();
+    print_circle_hash = NewHash();
     print_circle_count = 0;
     hanging_parens = 0;
     need_whitespace = 0;
