@@ -12,6 +12,8 @@
  * the naming of local variables, calling conventions, and so forth.
  * ----------------------------------------------------------------------------- */
 
+char cvsroot_cwrap_c[] = "$Id: cwrap.c 13935 2012-11-24 14:13:58Z wsfulton $";
+
 #include "swig.h"
 
 extern int cparse_cplusplus;
@@ -807,7 +809,7 @@ void Swig_replace_special_variables(Node *n, Node *parentnode, String *code) {
  * ----------------------------------------------------------------------------- */
 static String *extension_code(Node *n, const String *function_name, ParmList *parms, SwigType *return_type, const String *code, int cplusplus, const String *self) {
   String *parms_str = cplusplus ? ParmList_str_defaultargs(parms) : ParmList_str(parms);
-  String *sig = NewStringf("%s(%s)", function_name, (cplusplus || Len(parms_str)) ? parms_str : "void");
+  String *sig = NewStringf("%s(%s)", function_name, parms_str);
   String *rt_sig = SwigType_str(return_type, sig);
   String *body = NewStringf("SWIGINTERN %s", rt_sig);
   Printv(body, code, "\n", NIL);
@@ -853,9 +855,6 @@ int Swig_MethodToFunction(Node *n, const_String_or_char_ptr nspace, String *clas
   String *self = 0;
   int is_smart_pointer_overload = 0;
   String *qualifier = Getattr(n, "qualifier");
-  String *directorScope = NewString(nspace);
-
-  Replace(directorScope, NSPACE_SEPARATOR, "_", DOH_REPLACE_ANY);
   
   /* If smart pointer without const overload or mutable method, change self dereferencing */
   if (flags & CWRAP_SMART_POINTER) {
@@ -938,10 +937,7 @@ int Swig_MethodToFunction(Node *n, const_String_or_char_ptr nspace, String *clas
 	/* If protected access (can only be if a director method) then call the extra public accessor method (language module must provide this) */
 	String *explicit_qualifier_tmp = SwigType_namestr(Getattr(Getattr(parentNode(n), "typescope"), "qname"));
 	explicitcall_name = NewStringf("%sSwigPublic", name);
-        if (Len(directorScope) > 0)
-	  explicit_qualifier = NewStringf("SwigDirector_%s_%s", directorScope, explicit_qualifier_tmp);
-        else
-	  explicit_qualifier = NewStringf("SwigDirector_%s", explicit_qualifier_tmp);
+	explicit_qualifier = NewStringf("SwigDirector_%s", explicit_qualifier_tmp);
 	Delete(explicit_qualifier_tmp);
       } else {
 	explicit_qualifier = SwigType_namestr(Getattr(Getattr(parentNode(n), "typescope"), "qname"));
@@ -1061,7 +1057,6 @@ int Swig_MethodToFunction(Node *n, const_String_or_char_ptr nspace, String *clas
   Delete(p);
   Delete(self);
   Delete(parms);
-  Delete(directorScope);
   return SWIG_OK;
 }
 
@@ -1112,9 +1107,6 @@ int Swig_ConstructorToFunction(Node *n, const_String_or_char_ptr nspace, String
   ParmList *directorparms;
   SwigType *type;
   int use_director;
-  String *directorScope = NewString(nspace);
- 
-  Replace(directorScope, NSPACE_SEPARATOR, "_", DOH_REPLACE_ANY);
 
   use_director = Swig_directorclass(n);
 
@@ -1175,17 +1167,12 @@ int Swig_ConstructorToFunction(Node *n, const_String_or_char_ptr nspace, String
 	Node *parent = Swig_methodclass(n);
 	int abstract = Getattr(parent, "abstracts") != 0;
 	String *name = Getattr(parent, "sym:name");
-	String *directorname;
+	String *directorname = NewStringf("SwigDirector_%s", name);
 	String *action = NewStringEmpty();
 	String *tmp_none_comparison = Copy(none_comparison);
 	String *director_call;
 	String *nodirector_call;
 
-        if (Len(directorScope) > 0)
-          directorname = NewStringf("SwigDirector_%s_%s", directorScope, name);
-        else 
-          directorname = NewStringf("SwigDirector_%s", name);
-
 	Replaceall(tmp_none_comparison, "$arg", "arg1");
 
 	director_call = Swig_cppconstructor_director_call(directorname, directorparms);
@@ -1246,7 +1233,6 @@ int Swig_ConstructorToFunction(Node *n, const_String_or_char_ptr nspace, String
   if (directorparms != parms)
     Delete(directorparms);
   Delete(parms);
-  Delete(directorScope);
   return SWIG_OK;
 }
 
