@@ -13,6 +13,8 @@
  * are provided.
  * ----------------------------------------------------------------------------- */
 
+char cvsroot_include_c[] = "$Id: include.c 13940 2012-11-26 20:31:54Z wsfulton $";
+
 #include "swig.h"
 
 static List   *directories = 0;	        /* List of include directories */
@@ -163,8 +165,7 @@ static FILE *Swig_open_file(const_String_or_char_ptr name, int sysfile, int use_
   String *filename;
   List *spath = 0;
   char *cname;
-  int i, ilen, nbytes;
-  char bom[3];
+  int i, ilen;
 
   if (!directories)
     directories = NewList();
@@ -192,14 +193,6 @@ static FILE *Swig_open_file(const_String_or_char_ptr name, int sysfile, int use_
   if (f) {
     Delete(lastpath);
     lastpath = filename;
-
-    /* Skip the UTF-8 BOM if it's present */
-    nbytes = fread(bom, 1, 3, f);
-    if (nbytes == 3 && bom[0] == (char)0xEF && bom[1] == (char)0xBB && bom[2] == (char)0xBF) {
-      /* skip */
-    } else {
-      fseek(f, 0, SEEK_SET);
-    }
   }
   return f;
 }
