@@ -11,7 +11,7 @@
  * This file implements the SWIG symbol table.  See details below.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_symbol_c[] = "$Id: symbol.c 13505 2012-08-04 20:24:47Z wsfulton $";
+char cvsroot_symbol_c[] = "$Id: symbol.c 13882 2012-11-13 22:19:47Z wsfulton $";
 
 #include "swig.h"
 #include "swigwarn.h"
@@ -1549,6 +1549,7 @@ static int symbol_no_constructor(Node *n) {
  * Swig_symbol_type_qualify()
  *
  * Create a fully qualified type name
+ * Note: Does not resolve a constructor if passed in as the 'type'.
  * ----------------------------------------------------------------------------- */
 
 SwigType *Swig_symbol_type_qualify(const SwigType *t, Symtab *st) {
@@ -1800,7 +1801,7 @@ String *Swig_symbol_string_qualify(String *s, Symtab *st) {
   char *c = Char(s);
   int first_char = 1;
   while (*c) {
-    if (isalpha((int) *c) || (*c == '_') || (*c == ':') || (isdigit((int) *c) && !first_char)) {
+    if (isalpha((int) *c) || (*c == '_') || (*c == ':') || (*c == '~' && first_char) || (isdigit((int) *c) && !first_char)) {
       Putc(*c, id);
       have_id = 1;
     } else {
