@@ -12,7 +12,7 @@
  * parse trees.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_tree_c[] = "$Id: tree.c 13340 2012-07-21 22:52:30Z wsfulton $";
+char cvsroot_tree_c[] = "$Id: tree.c 13939 2012-11-26 20:30:37Z wsfulton $";
 
 #include "swig.h"
 #include <stdarg.h>
@@ -82,7 +82,7 @@ void Swig_print_node(Node *obj) {
       Printf(stdout, "%-12s - %s\n", k, ParmList_str_defaultargs(Getattr(obj, k)));
     } else {
       DOH *o;
-      char *trunc = "";
+      const char *trunc = "";
       print_indent(2);
       if (DohIsString(Getattr(obj, k))) {
 	o = Str(Getattr(obj, k));
