@@ -14,9 +14,10 @@
  * like typedef, namespaces, etc.
  * ----------------------------------------------------------------------------- */
 
+char cvsroot_typeobj_c[] = "$Id: typeobj.c 13903 2012-11-18 00:44:39Z wsfulton $";
+
 #include "swig.h"
 #include <ctype.h>
-#include <limits.h>
 
 /* -----------------------------------------------------------------------------
  * Synopsis
@@ -426,60 +427,61 @@ int SwigType_isreference(const SwigType *t) {
  * Repeated qualifications have no effect.  Moreover, the order of qualifications
  * is alphabetical---meaning that "const volatile" and "volatile const" are
  * stored in exactly the same way as "q(const volatile)".
- * 'qual' can be a list of multiple qualifiers in any order, separated by spaces.
  * ----------------------------------------------------------------------------- */
 
 SwigType *SwigType_add_qualifier(SwigType *t, const_String_or_char_ptr qual) {
-  List *qlist;
-  String *allq, *newq;
-  int i, sz;
-  const char *cqprev = 0;
-  const char *c = Char(t);
-  const char *cqual = Char(qual);
+  String *newq;
+  int sz, added = 0;
+  char *q, *cqual;
 
-  /* if 't' has no qualifiers and 'qual' is a single qualifier, simply add it */
-  if ((strncmp(c, "q(", 2) != 0) && (strstr(cqual, " ") == 0)) {
+  char *c = Char(t);
+  cqual = Char(qual);
+
+  if (!(strncmp(c, "q(", 2) == 0)) {
     String *temp = NewStringf("q(%s).", cqual);
     Insert(t, 0, temp);
     Delete(temp);
     return t;
   }
 
-  /* create string of all qualifiers */
-  if (strncmp(c, "q(", 2) == 0) {
-    allq = SwigType_parm(t);
-    Append(allq, " ");
-    SwigType_del_element(t);     /* delete old qualifier list from 't' */
-  } else {
-    allq = NewStringEmpty();
+  /* The type already has a qualifier on it.  In this case, we first check to
+     see if the qualifier is already specified.  In that case do nothing.
+     If it is a new qualifier, we add it to the qualifier list in alphabetical
+     order */
+
+  sz = element_size(c);
+
+  if (strstr(c, cqual)) {
+    /* Qualifier already added */
+    return t;
   }
-  Append(allq, qual);
 
-  /* create list of all qualifiers from string */
-  qlist = Split(allq, ' ', INT_MAX);
-  Delete(allq);
+  /* Add the qualifier to the existing list. */
 
-  /* sort in alphabetical order */
-  SortList(qlist, Strcmp);
-
-  /* create new qualifier string from unique elements of list */
-  sz = Len(qlist);
   newq = NewString("q(");
-  for (i = 0; i < sz; ++i) {
-    String *q = Getitem(qlist, i);
-    const char *cq = Char(q);
-    if (cqprev == 0 || strcmp(cqprev, cq) != 0) {
-      if (i > 0) {
-        Append(newq, " ");
-      }
+  q = c + 2;
+  q = strtok(q, " ).");
+  while (q) {
+    if (strcmp(cqual, q) < 0) {
+      /* New qualifier is less that current qualifier.  We need to insert it */
+      Append(newq, cqual);
+      Append(newq, " ");
       Append(newq, q);
-      cqprev = cq;
+      added = 1;
+    } else {
+      Append(newq, q);
+    }
+    q = strtok(NULL, " ).");
+    if (q) {
+      Append(newq, " ");
     }
   }
+  if (!added) {
+    Append(newq, " ");
+    Append(newq, cqual);
+  }
   Append(newq, ").");
-  Delete(qlist);
-
-  /* replace qualifier string with new one */
+  Delslice(t, 0, sz);
   Insert(t, 0, newq);
   Delete(newq);
   return t;
@@ -746,6 +748,7 @@ SwigType *SwigType_add_function(SwigType *t, ParmList *parms) {
 
   Insert(t, 0, ").");
   pstr = NewString("f(");
+  p = parms;
   for (p = parms; p; p = nextSibling(p)) {
     if (p != parms)
       Putc(',', pstr);
@@ -843,6 +846,7 @@ SwigType *SwigType_add_template(SwigType *t, ParmList *parms) {
   Parm *p;
 
   Append(t, "<(");
+  p = parms;
   for (p = parms; p; p = nextSibling(p)) {
     String *v;
     if (Getattr(p, "default"))
